@@ -79,10 +79,10 @@ class Client(Buildin):
         return self._loop
 
     def is_connected(self) -> bool:
-        """Can be used to check if the client is conected.
+        """Can be used to check if the client is connected.
 
         Returns:
-            bool: True when the client is connected else False.
+            bool: `True` when the client is connected else `False`.
         """
         return bool(self._protocol and self._protocol.transport)
 
@@ -139,20 +139,24 @@ class Client(Buildin):
         addr, port = socket.getpeername()
         return f'{addr}:{port}'
 
-    async def connect_pool(self, pool: list, *auth: Union[str, tuple]) -> None:
+    def connect_pool(
+            self,
+            pool: list,
+            *auth: Union[str, tuple]
+    ) -> asyncio.Future:
         """Connect using a connection pool.
 
         When using a connection pool, the client will randomly choose a node
         to connect to. When a node is going down, it will inform the client
-        so it will automitically re-connect to another node. Connections will
+        so it will automatically re-connect to another node. Connections will
         automatically authenticate so the connection pool requires credentials
-        to perfrom the authentication.
+        to perform the authentication.
 
         Examples:
             >>> await connect_pool([
                 'node01.local',             # address as string
                 'node02.local',             # port will default to 9200
-                ('node03.local', 9201),     # ..or with an eplicit port
+                ('node03.local', 9201),     # ..or with an explicit port
             ], "admin", "pass")
 
         Args:
@@ -164,6 +168,11 @@ class Client(Buildin):
                 tuple with username and password. (the latter may be provided
                 as two separate arguments
 
+        Returns:
+            asyncio.Future (None):
+                Future which should be awaited. The result of the future will
+                be set to `None` when successful.
+
         Remarks:
             Do not use this method if the client is already
             connected. This can be checked with `client.is_connected()`.
@@ -177,14 +186,14 @@ class Client(Buildin):
             for address in pool))
         self._auth = self._auth_check(auth)
         self._pool_idx = random.randint(0, len(pool) - 1)
-        await self.reconnect()
+        return self.reconnect()
 
-    async def connect(
+    def connect(
             self,
             host: str,
             port: int = 9200,
             timeout: Optional[int] = 5
-    ) -> None:
+    ) -> asyncio.Future:
         """Connect to ThingsDB.
 
         This method will *only* create a connection, so the connection is not
@@ -200,9 +209,14 @@ class Client(Buildin):
                 Defaults to 9200.
             timeout (int, optional):
                 Can be be used to control the maximum time the client will
-                attempt to create a connection. The timeout may be set to None
-                in which case the client will wait forever on a response.
-                Defaults to 5.
+                attempt to create a connection. The timeout may be set to
+                `None` in which case the client will wait forever on a
+                response. Defaults to 5.
+
+        Returns:
+            asyncio.Future (None):
+                Future which should be awaited. The result of the future will
+                be set to `None` when successful.
 
         Remarks:
             Do not use this method if the client is already
@@ -211,7 +225,7 @@ class Client(Buildin):
         assert self.is_connected() is False
         self._pool = ((host, port),)
         self._pool_idx = 0
-        await self._connect(timeout=timeout)
+        return self._connect(timeout=timeout)
 
     async def reconnect(self) -> None:
         """Re-connect to ThingsDB.
@@ -230,15 +244,19 @@ class Client(Buildin):
             self._reconnecting = False
 
     async def wait_closed(self) -> None:
-        """Wait for a connecion to close.
+        """Wait for a connection to close.
 
-        Can be used after calliing the `close()` method to determine when the
+        Can be used after calling the `close()` method to determine when the
         connection is actually closed.
         """
         if self._protocol and self._protocol.close_future:
             await self._protocol.close_future
 
-    async def authenticate(self, *auth, timeout: Optional[int] = 5) -> None:
+    async def authenticate(
+            self,
+            *auth: Union[str, tuple],
+            timeout: Optional[int] = 5
+    ) -> None:
         """Authenticate a ThingsDB connection.
 
         Args:
@@ -249,7 +267,7 @@ class Client(Buildin):
             timeout (int, optional):
                 Can be be used to control the maximum time in seconds for the
                 client to wait for response on the authentication request.
-                The timeout may be set to None in which case the client will
+                The timeout may be set to `None` in which case the client will
                 wait forever on a response. Defaults to 5.
         """
         if len(auth) == 1:
@@ -260,14 +278,14 @@ class Client(Buildin):
         if self._reconnect:
             await self.watch(scope='@n')
 
-    async def query(
+    def query(
             self,
             code: str,
             scope: Optional[str] = None,
             timeout: Optional[int] = None,
             convert_vars: bool = True,
             **kwargs: Any
-    ) -> Any:
+    ) -> asyncio.Future:
         """Query ThingsDB.
 
         Use this method to run `code` in a scope.
@@ -282,14 +300,14 @@ class Client(Buildin):
             timeout (int, optional):
                 Raise a time-out exception if no response is received within X
                 seconds. If no time-out is given, the client will wait forever.
-                Defaults to None.
+                Defaults to `None`.
             convert_vars (bool, optional):
-                Only applicable if `**kwargs` are given. If set to True, then
+                Only applicable if `**kwargs` are given. If set to `True`, then
                 the provided **kwargs values will be converted so ThingsDB can
                 understand them. For example, a thing should be given just by
                 it's ID and with conversion the `#` will be extracted. When
-                this argument is False, the **kwargs stay untouched.
-                Defaults to True.
+                this argument is `False`, the **kwargs stay untouched.
+                Defaults to `True`.
             **kwargs (any, optional):
                 Can be used to inject variable into the ThingsDB code.
 
@@ -302,8 +320,10 @@ class Client(Buildin):
                 'title': 'Manual ThingsDB'
             })
 
-        Retuns:
-            result (any): The result of the ThingsDB code.
+        Returns:
+            asyncio.Future (any):
+                Future which should be awaited. The result of the future will
+                contain the result of the ThingsDB code when successful.
 
         Remarks:
             If the ThingsDB code will return with an exception, then this
@@ -324,17 +344,16 @@ class Client(Buildin):
         else:
             data = [scope, code]
 
-        future = self._protocol.write(Proto.REQ_QUERY, data, timeout=timeout)
-        return await future
+        return self._protocol.write(Proto.REQ_QUERY, data, timeout=timeout)
 
-    async def run(
+    def run(
             self,
             procedure: str,
             *args: Optional[Any],
             scope: Optional[str] = None,
             timeout: Optional[int] = None,
             convert_args: bool = True
-    ) -> Any:
+    ) -> asyncio.Future:
         """Run a procedure.
 
         Use this method to run a stored procedure in a scope.
@@ -353,17 +372,19 @@ class Client(Buildin):
             timeout (int, optional):
                 Raise a time-out exception if no response is received within X
                 seconds. If no time-out is given, the client will wait forever.
-                Defaults to None.
+                Defaults to `None`.
             convert_args (bool, optional):
-                Only applicable if `*args` are given. If set to True, then
-                the provided *args values will be converted so ThingsDB can
+                Only applicable if `*args` are given. If set to `True`, then
+                the provided `*args` values will be converted so ThingsDB can
                 understand them. For example, a thing should be given just by
                 it's ID and with conversion the `#` will be extracted. When
-                this argument is False, the **kwargs stay untouched.
-                Defaults to True.
+                this argument is `False`, the `*args` stay untouched.
+                Defaults to `True`.
 
-        Retuns:
-            result (any): The result of the ThingsDB procedure.
+        Returns:
+            asyncio.Future (any):
+                Future which should be awaited. The result of the future will
+                contain the result of the ThingsDB procedure when successful.
 
         Remarks:
             If the ThingsDB code will return with an exception, then this
@@ -379,15 +400,13 @@ class Client(Buildin):
 
         arguments = (convert(arg) for arg in args) if convert_args else args
 
-        future = self._protocol.write(
+        return self._protocol.write(
             Proto.REQ_RUN,
             [scope, procedure, *arguments],
             timeout=timeout)
 
-        return await future
-
-    def watch(self, *ids: int, scope: Optional[str] = None) -> None:
-        """Subscibe for changes on given things.
+    def watch(self, *ids: int, scope: Optional[str] = None) -> asyncio.Future:
+        """Subscribe for changes on given things.
 
         This method accepts one or more thing ids to subscribe to. This
         method will simply return None as soon as the subscribe request is
@@ -409,6 +428,10 @@ class Client(Buildin):
                 things so only collection scopes can be used.
                 See https://docs.thingsdb.net/v0/overview/scopes/ for how to
                 format a scope.
+
+        Returns:
+            asyncio.Future (None):
+                Future which result will be set to `None` if successful.
         """
         if self._protocol is None:
             raise ConnectionError('no connection')
@@ -418,7 +441,11 @@ class Client(Buildin):
 
         return self._protocol.write(Proto.REQ_WATCH, [scope, *ids])
 
-    def unwatch(self, *ids: int, scope: Optional[str] = None):
+    def unwatch(
+            self,
+            *ids: int,
+            scope: Optional[str] = None
+    ) -> asyncio.Future:
         """Unsubscribe for changes on given things.
 
         Stop receiving events for the things given by one or more ids. It is
@@ -436,6 +463,10 @@ class Client(Buildin):
                 things so only collection scopes can be used.
                 See https://docs.thingsdb.net/v0/overview/scopes/ for how to
                 format a scope.
+
+        Returns:
+            asyncio.Future (None):
+                Future which result will be set to `None` if successful.
         """
         if self._protocol is None:
             raise ConnectionError('no connection')
@@ -528,11 +559,11 @@ class Client(Buildin):
         for event_handler in self._event_handlers:
             event_handler.on_reconnect()
 
-    async def _authenticate(self, timeout):
+    def _authenticate(self, timeout):
         if self._protocol is None:
             raise ConnectionError('no connection')
 
-        await self._protocol.write(
+        return self._protocol.write(
             Proto.REQ_AUTH,
             data=self._auth,
             timeout=timeout)
